#!/usr/bin/env python
# encoding: utf-8
'''
CommandLineSample -- sample python application for WAPI

CommandLineSample is a command line sample python application for WAPI.

It defines:
 Method print_help - show the help message and exit;
 Method perform_request - send an HTTPS request to the Web API server;
 Method Main - work with command line options and send an HTTPS
request to the WAPI.

@copyright:   Copyright 2012 Infoblox Inc. All rights reserved.

@license:

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY INFOBLOX AND CONTRIBUTORS ``AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INFOBLOX OR CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are
those of the authors and should not be interpreted as representing official
policies, either expressed or implied, of Infoblox.

'''

import getopt
import sys

from httplib import HTTPSConnection

operations = dict([('create', 'POST'),
                   ('update', 'PUT'),
                   ('read', 'GET'),
                   ('readall', 'GET'),
                   ('delete', 'DELETE'),
                   ('search', 'GET')])


def print_help():
    print '''
    Usage: wapiclient.py [options]

CommandLineSample is a command line sample python client for WAPI.

Copyright 2012 Infoblox

Options:
  -h, --help            show this help message and exit
  -p PASSWORD, --password=PASSWORD
                        Required argument: User password for the
                        authentication.
  -u USERNAME, --username=USERNAME
                        Required argument: User name for the authentication.
  -a ADDRESS, --host_address=ADDRESS
                        Required argument: Web API service host address.
  -v VERSION, --version=VERSION
                        Optional argument: Version of WAPI service. Default is
                        1.0.
  -t OBJECTTYPE, --objecttype=OBJECT_TYPE
                        Required argument: Type of Wapi object.
  -r REF, --ref=REF     Optional argument: Ref to the Wapi object.
  -d DATA, --data=DATA  Optional argument: input data such as search criteria
                        or fields of new object.
  -o OPERATION, --operation=OPERATION
                        Required argument: Operation with WAPI objects:
                        create, read, readall, update, delete or search.
'''


def perform_request(address, username, password, version, objecttype, ref, \
                    data, operation):
    '''
    Send an HTTPS request to the Web API server.
    '''
    if operation in operations:
        operation = operations[operation]
    else:
        raise ValueError('Invalid operation: ' + operation)

    if ref == '':
        ref = objecttype
    path = '/wapi/v' + version + '/' + ref
    # Creating HTTPS connection.
    # Please note that this class does not perform any verification of
    # the server's certificate.
    conn = HTTPSConnection(address)
    auth_header = 'Basic %s' % \
            (':'.join([username, password]).encode('Base64').strip('\r\n'))

    conn.request(operation, path, data, \
                 {'Authorization':auth_header, \
                  'Content-Type':'application/x-www-form-urlencoded'})
    response = conn.getresponse().read()
    return response


def main(argv=None):
    '''
    Main function: work with command line options and send an HTTPS request to the WAPI.
    '''

    try:
        opts, args = getopt.getopt(sys.argv[1:], 'ha:u:p:v:t:r:d:o:',
                                   ['help', 'address=', 'username=',
                                    'password=', 'version=', 'objecttype=',
                                    'ref=', 'data=', 'operation='])
    except getopt.GetoptError, err:
        # Print help information and exit:
        print str(err)
        print_help()
        sys.exit(2)

    # Initialization optional parameters.
    ref = ''
    data = ''
    version = '1.0'

    # Parse command line options
    for opt, arg in opts:
        if opt in ('-h', '--help'):
            print_help()
            sys.exit()
        elif opt in ('-a', '--address'):
            address = arg
        elif opt in ('-u', '--username'):
            username = arg
        elif opt in ('-p', '--password'):
            password = arg
        elif opt in ('-v', '--version'):
            version = arg
        elif opt in ('-t', '--objecttype'):
            objecttype = arg
        elif opt in ('-r', '--ref'):
            ref = arg
        elif opt in ('-d', '--data'):
            data = arg
        elif opt in ('-o', '--operation'):
            operation = arg

    print perform_request(address, username, password, version, objecttype, \
                          ref, data, operation)


if __name__ == '__main__':
    sys.exit(main())
